#include "float.h"
#include "math.h"
#include "dsk6713.h"
#include "dsk6713_aic23.h"

#include <stdio.h>
#include <csl_timer.h>
#include <csl_irq.h>
#include <csl_emif.h>
#include <csl_gpio.h>

extern far void vectors();
static TIMER_Handle hTimer1,hTimer0;
static GPIO_Handle hGpio;  

     
static Uint32 TimerControl=0x00000200; //Valore del registro CTL del Timer1
static MCBSP_Config Data={  	
  							0xC10000,
    						0xA0,
    						0xA0,
    						0x00,
    						0x00,
    						0x00,
    						0x00,
    						0x03
							};

DSK6713_AIC23_Config config = {   //Variabile struct tipo DSK6713_AIC23_Config per la config del DSP
    0x0117, // 0 DSK6713_AIC23_LEFTINVOL  Left line input channel volume
    0x0117, // 1 DSK6713_AIC23_RIGHTINVOL Right line input channel volume
    0x00D8, // 2 DSK6713_AIC23_LEFTHPVOL  Left channel headphone volume
    0x00D8, // 3 DSK6713_AIC23_RIGHTHPVOL Right channel headphone volume
    0x0011, // 4 DSK6713_AIC23_ANAPATH    Analog audio path control
    0x0000, // 5 DSK6713_AIC23_DIGPATH    Digital audio path control
    0x0000, // 6 DSK6713_AIC23_POWERDOWN  Power down control
    0x0043, // 7 DSK6713_AIC23_DIGIF      Digital audio interface format
    0x000D, // 8 DSK6713_AIC23_SAMPLERATE Sample rate control     8000 Hz
    0x0001  // 9 DSK6713_AIC23_DIGACT     Digital interface activation
};

static GPIO_Config conf={

0x00000000,    // gpgc
0x00000070,    // gpen
0x00000000,    // gpdir
0x00000000,    // gpva
0x00000000,    // gphm
0x00000000,    // gplm   
0x00000000,    // gppol
};



//Variabili di Controllo   
Uint8 Start_Per=0,k1=0,k2=0,Start_Frame=0,Start_Trans=0,Enab_Count=0,En_Calcol=0,Enab_Sound=0; 
Uint32 Supp=0,Periodo=0,Periodo_old=0,Supp_Cont=0;
Uint16 Ntot=0;

//Variabili Audio
float SampF=0,pi=3.141592653,fo=1000.0,Fs=8000.0;
static Uint32 sample=0;
Uint16 temp,n=0,i=0,Nmax=0;

//Variabili Distanza
Uint32 Tempo_Volo=0,Supp_Volo=0;
Uint32 OFFSET=26239,STEP=2952,THR=1405; 

void main()
{
   DSK6713_AIC23_CodecHandle hCodec;    
    
    
   DSK6713_init();   
   CSL_init();   
   IRQ_globalDisable();       // Globally disable interrupts  
      
   

   		//Configurazione GPIO
   GPIO_config(hGpio,&conf);


  		 //Configurazione Timer1   
   hTimer1 = TIMER_open(TIMER_DEV1, TIMER_OPEN_RESET);    
   TIMER_configArgs(hTimer1, TimerControl, 0xFEFFFFFF, 0x00000000 );    
   

   		//Configurazione MCBSP
   hCodec = DSK6713_AIC23_openCodec(0, &config);       
   MCBSP_config(_MCBSP_hDev1,&Data);           
   

   		//Configurazione Interrupts   
   IRQ_setVecs(vectors);     // point to the IRQ vector table          
   IRQ_nmiEnable();          // Enable NMI interrupt
   
   IRQ_reset(14);   
   IRQ_enable(14);  //Abilito l'interruzione del McBSP in modalit speciificata W  
   
   IRQ_reset(4);   
   IRQ_enable(4);

   IRQ_reset(5);   
   IRQ_enable(5);

   IRQ_reset(6);   
   IRQ_enable(6);
   
   IRQ_globalEnable();       // Globally enable interrupts  
   
   
   //TIMER_start(hTimer1);      // Start the timers
   while (1);    
      
     
}


interrupt void
c_int04(void)    //Int04  l'interruzione del GPIO Input 4 sensibile al fronte di salita - TRASMETTITORE

{      
    
  if (Start_Frame==0)                                //arriva l'impulso=>parte il conteggio
     {
      Supp_Cont=TIMER_getCount(hTimer1);             //leggo il valore dello sfasamento tra quando  arrivato l'impulso e il fronte di
      						                         //salita 40KHz del periodo che  arrivato per ultimo. Serve per eliminare l'incertezza 
                                                     //(di ampiezza un periodo) della lettura del primo periodo che dipende da dove arriva 
						                             //l'impulso rispetto al periodo a 40 kHz. Arrivato l'impulso, inizio ad accumulare
	                                                 //dal fronte di salita successivo all'arrivo dell'impulso. In questo modo per perderei
	                                                 //il tempo trascorso tra l'arrivo del impulso e il primo fronte di salita che fa partire
	                                                 //l'accumulo dei vari periodi. Per evitare ci, al primo periodo letto sommo Supp_Cont
	                                                 //che  proprio la quantit che altrimenti perderei ***
      Start_Frame=1;                                 //abilito l'accumulo periodi
      Tempo_Volo=1470-Supp_Cont;                     //vedi *** sopra
      Enab_Count=1;                                  //abilito l'accumulo periodi per la prima volta in quel tempo di volo
     }
  
 //  {175, 196, 220, 247, 262, 294, 330, 349, 392, 440, 494, 523, 587, 659, 698, 784, 880, 988, 1046}; 
  
  if      ((Supp_Volo>=OFFSET)       &&(Supp_Volo<OFFSET+  STEP)) fo=175;
  else if ((Supp_Volo>=OFFSET+STEP)  &&(Supp_Volo<OFFSET+2*STEP)) fo=196;
  else if ((Supp_Volo>=OFFSET+2*STEP)&&(Supp_Volo<OFFSET+3*STEP)) fo=220;
  else if ((Supp_Volo>=OFFSET+3*STEP)&&(Supp_Volo<OFFSET+4*STEP)) fo=247;
  else if ((Supp_Volo>=OFFSET+4*STEP)&&(Supp_Volo<OFFSET+5*STEP)) fo=262;
  else if ((Supp_Volo>=OFFSET+5*STEP)&&(Supp_Volo<OFFSET+6*STEP)) fo=294;
  else if ((Supp_Volo>=OFFSET+6*STEP)&&(Supp_Volo<OFFSET+7*STEP)) fo=330;
  else if ((Supp_Volo>=OFFSET+7*STEP)&&(Supp_Volo<OFFSET+8*STEP)) fo=349;
  else if ((Supp_Volo>=OFFSET+8*STEP)&&(Supp_Volo<OFFSET+9*STEP)) fo=392;
  else if ((Supp_Volo>=OFFSET+9*STEP)&&(Supp_Volo<OFFSET+10*STEP)) fo=440;
  else if ((Supp_Volo>=OFFSET+10*STEP)&&(Supp_Volo<OFFSET+11*STEP)) fo=494;
  else if ((Supp_Volo>=OFFSET+11*STEP)&&(Supp_Volo<OFFSET+12*STEP)) fo=523; 
  else if ((Supp_Volo>=OFFSET+12*STEP)&&(Supp_Volo<OFFSET+13*STEP)) fo=587; 
  else if ((Supp_Volo>=OFFSET+13*STEP)&&(Supp_Volo<OFFSET+14*STEP)) fo=659; 
  else if ((Supp_Volo>=OFFSET+14*STEP)&&(Supp_Volo<OFFSET+15*STEP)) fo=698;   
  else if ((Supp_Volo>=OFFSET+15*STEP)&&(Supp_Volo<OFFSET+16*STEP)) fo=784; 
  else if ((Supp_Volo>=OFFSET+16*STEP)&&(Supp_Volo<OFFSET+17*STEP)) fo=880;   
  else if ((Supp_Volo>=OFFSET+17*STEP)&&(Supp_Volo<OFFSET+18*STEP)) fo=988; 
  else if ((Supp_Volo>=OFFSET+18*STEP)&&(Supp_Volo<OFFSET+19*STEP)) fo=1046;
  
     
   return;              
}    // end c_int04







interrupt void
c_int05(void)    //Int05  l'interruzione del GPIO Input 5 sensibile al fronte di salita - RICEVITORE

{     

   if (Start_Per==0) //Start_per  zero solo una volta all'inizio: serve per far parire il timer con l'istruzione giusta la prima volta (TIMER_start)
      {
       TIMER_start(hTimer1);
       Start_Per=1;
      }
   else if (Start_Per==1)  //eseguito sempre, tranne la prima volta
   	  {
       Periodo_old=Periodo;
       Periodo=TIMER_getCount(hTimer1);
       
       TIMER_setCount(hTimer1,0);            
      
       
       
       if (Start_Frame==1)  //arrivato l'impulso=>inizio ad accumulare e monitorare in attesa dell'attraversamento della soglia
          { 
           if ((Periodo_old>THR)&&(Periodo<THR))   //avvenuta la transizione                 
             {
             Start_Frame=0;  //chiudo il Frame=>non accumulo pi

             Supp_Volo=Tempo_Volo;  //variabile copia per valutare la distanza
             }     
       
          if (Enab_Count==0) Tempo_Volo+=Periodo; //accumulo periodi
          Enab_Count=0; 

          }   //End  if (Start_Frame==1)           
       
      } //End  if (Start_Per==1)
       
   if (En_Calcol==1)                 //non eseguo il conto a ogni transizione di salita del segnale a 40kHz, ma solo dopo aver mandato il segnale al MCBSP
	                                 //quando devo calcolare un valore nuovo: non spreco tempo inutilmente a calcolare un valore che ho gi calcolato
       {
        SampF=15000*sin(2*pi*fo/Fs*n); 
        temp=(Uint16)SampF;
        sample=temp;   
        sample<<= 16;     
        sample+=temp;      
        Nmax=(unsigned int)(Fs/fo);  
        En_Calcol=0;            
       
       }

   return;              
}    // end c_int05

interrupt void
c_int06(void)    //Int06  l'interruzione del GPIO Input 6 sensibile al fronte di salita

{
  Enab_Sound=1;  //premo il tasto=>abilito il suono(vedi interrupt MCBSP)
  n=0;           // l'indice temporale discreto dei campioni
  Ntot=0;        //quanti campioni audio butto fuori ad ogni pressione del tasto
}

interrupt void
c_int11(void)    //Int11  l'interruzione del McBSP1 XINT mappata sull'interrupt selector

{      
    
    MCBSP_write(_MCBSP_hDev1, sample);
    if (Enab_Sound==1) n++;                   //se il suono  abilitato, incremento l'indice temporale dei campioni per il calcolo del seno successivo
    if (n>Nmax) {Ntot+=n; n=0;}               //Nmax  il numero di campioni, campionati a 8kHz, che stanno nel periodo generico 1/fo della nota.
	                                          //finito il periodo azzero l'indice discreto temporale n
    En_Calcol=1;                              //abilito il calcolo del nuovo camoione sinusoidale con n nouvo
    if (Ntot>2000){ Enab_Sound=0; sample=0;}  //Se ho mandato fuori tutti i campioni stabiliti (2000=>0.25 secondi di suono)
	                                          //disabilito il suono e imposto sample=0
      
   return;              
}    // end c_int11
